@@ -16,6 +16,7 @@
 package org.gradle.api.artifacts.component;
 
 import org.gradle.api.Incubating;
+import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.internal.scan.UsedByScanPlugin;
 
 /**
@@ -49,5 +50,15 @@ public interface ModuleComponentIdentifier extends ComponentIdentifier {
      * @since 1.10
      */
     String getVersion();
+
+    /**
+     * The module identifier of the component. Returns the same information
+     * as {@link #getGroup()} and {@link #getModule()}.
+     *
+     * @return the module identifier
+     *
+     * @since 4.9
+     */
+    ModuleIdentifier getModuleIdentifier();
 }
 
