@@ -16,10 +16,12 @@
 
 package org.gradle.api.execution.internal;
 
+import com.google.common.collect.ImmutableMap;
 import org.gradle.api.internal.TaskInternal;
 import org.gradle.internal.execution.ExecuteTaskBuildOperationType;
+import org.gradle.internal.operations.trace.CustomOperationTraceSerialization;
 
-public class ExecuteTaskBuildOperationDetails implements ExecuteTaskBuildOperationType.Details {
+public class ExecuteTaskBuildOperationDetails implements ExecuteTaskBuildOperationType.Details, CustomOperationTraceSerialization {
 
     private final TaskInternal task;
 
@@ -53,4 +55,14 @@ public class ExecuteTaskBuildOperationDetails implements ExecuteTaskBuildOperati
         return task.getTaskIdentity().type;
     }
 
+    @Override
+    public Object getCustomOperationTraceSerializableModel() {
+        ImmutableMap.Builder<String, Object> builder = new ImmutableMap.Builder<String, Object>();
+        builder.put("buildPath", getBuildPath());
+        builder.put("taskPath", getTaskPath());
+        builder.put("taskClass", getTaskClass().getName());
+        builder.put("taskId", getTaskId());
+        return builder.build();
+
+    }
 }
