@@ -23,6 +23,8 @@ import org.gradle.api.internal.plugins.PluginAwareInternal;
 import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.invocation.Gradle;
 import org.gradle.execution.TaskExecutionGraphInternal;
+import org.gradle.internal.build.BuildState;
+import org.gradle.internal.build.PublicBuildPath;
 import org.gradle.internal.scan.UsedByScanPlugin;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
@@ -46,6 +48,11 @@ public interface GradleInternal extends Gradle, PluginAwareInternal {
 
     GradleInternal getRoot();
 
+    /**
+     * Returns the {@link BuildState} that manages the state of this instance.
+     */
+    BuildState getOwner();
+
     /**
      * {@inheritDoc}
      */
@@ -123,4 +130,6 @@ public interface GradleInternal extends Gradle, PluginAwareInternal {
     void setIdentityPath(Path path);
 
     String contextualize(String description);
+
+    PublicBuildPath getPublicBuildPath();
 }
