@@ -17,6 +17,8 @@
 package org.gradle.api.internal.changedetection.state;
 
 import net.rubygrapefruit.platform.ThreadSafe;
+import org.gradle.api.internal.changedetection.state.mirror.PhysicalSnapshot;
+import org.gradle.internal.file.FileMetadataSnapshot;
 
 import javax.annotation.Nullable;
 
@@ -28,17 +30,12 @@ import javax.annotation.Nullable;
 @ThreadSafe
 public interface FileSystemMirror {
     @Nullable
-    FileSnapshot getFile(String path);
+    PhysicalSnapshot getSnapshot(String absolutePath);
 
-    void putFile(FileSnapshot file);
+    void putSnapshot(PhysicalSnapshot file);
 
     @Nullable
-    Snapshot getContent(String path);
+    FileMetadataSnapshot getMetadata(String absolutePath);
 
-    void putContent(String path, Snapshot snapshot);
-
-    @Nullable
-    FileTreeSnapshot getDirectoryTree(String path);
-
-    void putDirectory(FileTreeSnapshot directory);
+    void putMetadata(String absolutePath, FileMetadataSnapshot stat);
 }
