@@ -18,6 +18,7 @@ package org.gradle.api.tasks;
 import groovy.lang.Closure;
 import org.gradle.api.Action;
 import org.gradle.api.GradleException;
+import org.gradle.api.NonNullApi;
 import org.gradle.api.Task;
 import org.gradle.api.Transformer;
 import org.gradle.api.file.CopySpec;
@@ -30,6 +31,7 @@ import org.gradle.api.internal.ClosureBackedAction;
 import org.gradle.api.internal.ConventionTask;
 import org.gradle.api.internal.file.FileLookup;
 import org.gradle.api.internal.file.FileResolver;
+import org.gradle.api.internal.file.collections.DirectoryFileTreeFactory;
 import org.gradle.api.internal.file.copy.ClosureBackedTransformer;
 import org.gradle.api.internal.file.copy.CopyAction;
 import org.gradle.api.internal.file.copy.CopyActionExecuter;
@@ -40,8 +42,8 @@ import org.gradle.api.internal.file.copy.DefaultCopySpec;
 import org.gradle.api.specs.Spec;
 import org.gradle.internal.nativeplatform.filesystem.FileSystem;
 import org.gradle.internal.reflect.Instantiator;
-import org.gradle.util.DeprecationLogger;
 
+import javax.annotation.Nullable;
 import javax.inject.Inject;
 import java.io.FilterReader;
 import java.util.Map;
@@ -52,10 +54,9 @@ import java.util.regex.Pattern;
 /**
  * {@code AbstractCopyTask} is the base class for all copy tasks.
  */
+@NonNullApi
 public abstract class AbstractCopyTask extends ConventionTask implements CopySpec, CopySpecSource {
 
-    private static final String CONFIGURE_SPEC_DURING_CONFIGURATION = "Consider configuring the spec during configuration time, or using a separate task to do the configuration";
-
     private final CopySpecInternal rootSpec;
     private final CopySpecInternal mainSpec;
 
@@ -65,15 +66,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
             @Override
             public void childSpecAdded(CopySpecInternal.CopySpecAddress path, final CopySpecInternal spec) {
                 if (getState().getExecuting()) {
-                    if (getProject().getGradle().getStartParameter().isBuildCacheEnabled() && getState().getTaskOutputCaching().isEnabled()) {
-                        throw new GradleException("It is not allowed to modify child specs of the task at execution time when task output caching is enabled. "
-                            + CONFIGURE_SPEC_DURING_CONFIGURATION + ".");
-                    }
-                    DeprecationLogger.nagUserOfDeprecated(
-                        "Configuring child specs of a copy task at execution time of the task",
-                        CONFIGURE_SPEC_DURING_CONFIGURATION
-                    );
-                    return;
+                    throw new GradleException("You cannot add child specs at execution time. Consider configuring this task during configuration time or using a separate task to do the configuration.");
                 }
 
                 StringBuilder specPropertyNameBuilder = new StringBuilder("rootSpec");
@@ -83,7 +76,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
 
                 getInputs().files(new Callable<FileTree>() {
                     @Override
-                    public FileTree call() throws Exception {
+                    public FileTree call() {
                         return resolver.getSource();
                     }
                 })
@@ -93,43 +86,45 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
 
                 getInputs().property(specPropertyName + ".destPath", new Callable<String>() {
                     @Override
-                    public String call() throws Exception {
+                    public String call() {
                         return resolver.getDestPath().getPathString();
                     }
                 });
                 getInputs().property(specPropertyName + ".caseSensitive", new Callable<Boolean>() {
                     @Override
-                    public Boolean call() throws Exception {
+                    public Boolean call() {
                         return spec.isCaseSensitive();
                     }
                 });
                 getInputs().property(specPropertyName + ".includeEmptyDirs", new Callable<Boolean>() {
                     @Override
-                    public Boolean call() throws Exception {
+                    public Boolean call() {
                         return spec.getIncludeEmptyDirs();
                     }
                 });
                 getInputs().property(specPropertyName + ".duplicatesStrategy", new Callable<DuplicatesStrategy>() {
                     @Override
-                    public DuplicatesStrategy call() throws Exception {
+                    public DuplicatesStrategy call() {
                         return spec.getDuplicatesStrategy();
                     }
                 });
                 getInputs().property(specPropertyName + ".dirMode", new Callable<Integer>() {
+                    @Nullable
                     @Override
-                    public Integer call() throws Exception {
+                    public Integer call() {
                         return spec.getDirMode();
                     }
-                });
+                }).optional(true);
                 getInputs().property(specPropertyName + ".fileMode", new Callable<Integer>() {
+                    @Nullable
                     @Override
-                    public Integer call() throws Exception {
+                    public Integer call() {
                         return spec.getFileMode();
                     }
-                });
+                }).optional(true);
                 getInputs().property(specPropertyName + ".filteringCharset", new Callable<String>() {
                     @Override
-                    public String call() throws Exception {
+                    public String call() {
                         return spec.getFilteringCharset();
                     }
                 });
@@ -172,6 +167,11 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
         throw new UnsupportedOperationException();
     }
 
+    @Inject
+    protected DirectoryFileTreeFactory getDirectoryFileTreeFactory() {
+        throw new UnsupportedOperationException();
+    }
+
     @TaskAction
     protected void copy() {
         CopyActionExecuter copyActionExecuter = createCopyActionExecuter();
@@ -197,6 +197,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     }
 
     @Internal
+    @Override
     public CopySpecInternal getRootSpec() {
         return rootSpec;
     }
@@ -214,6 +215,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
      * {@inheritDoc}
      */
     @Internal
+    @Override
     public boolean isCaseSensitive() {
         return getMainSpec().isCaseSensitive();
     }
@@ -221,6 +223,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setCaseSensitive(boolean caseSensitive) {
         getMainSpec().setCaseSensitive(caseSensitive);
     }
@@ -229,6 +232,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
      * {@inheritDoc}
      */
     @Internal
+    @Override
     public boolean getIncludeEmptyDirs() {
         return getMainSpec().getIncludeEmptyDirs();
     }
@@ -236,6 +240,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setIncludeEmptyDirs(boolean includeEmptyDirs) {
         getMainSpec().setIncludeEmptyDirs(includeEmptyDirs);
     }
@@ -243,7 +248,8 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
-    public void setDuplicatesStrategy(DuplicatesStrategy strategy) {
+    @Override
+    public void setDuplicatesStrategy(@Nullable DuplicatesStrategy strategy) {
         getRootSpec().setDuplicatesStrategy(strategy);
     }
 
@@ -251,6 +257,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
      * {@inheritDoc}
      */
     @Internal
+    @Override
     public DuplicatesStrategy getDuplicatesStrategy() {
         return getRootSpec().getDuplicatesStrategy();
     }
@@ -258,6 +265,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask from(Object... sourcePaths) {
         getMainSpec().from(sourcePaths);
         return this;
@@ -266,6 +274,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask filesMatching(String pattern, Action<? super FileCopyDetails> action) {
         getMainSpec().filesMatching(pattern, action);
         return this;
@@ -274,6 +283,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask filesMatching(Iterable<String> patterns, Action<? super FileCopyDetails> action) {
         getMainSpec().filesMatching(patterns, action);
         return this;
@@ -282,6 +292,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask filesNotMatching(String pattern, Action<? super FileCopyDetails> action) {
         getMainSpec().filesNotMatching(pattern, action);
         return this;
@@ -290,6 +301,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask filesNotMatching(Iterable<String> patterns, Action<? super FileCopyDetails> action) {
         getMainSpec().filesNotMatching(patterns, action);
         return this;
@@ -298,6 +310,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask from(Object sourcePath, final Closure c) {
         getMainSpec().from(sourcePath, new ClosureBackedAction<CopySpec>(c));
         return this;
@@ -306,6 +319,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask from(Object sourcePath, Action<? super CopySpec> configureAction) {
         getMainSpec().from(sourcePath, configureAction);
         return this;
@@ -314,6 +328,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public CopySpec with(CopySpec... sourceSpecs) {
         getMainSpec().with(sourceSpecs);
         return this;
@@ -322,6 +337,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask into(Object destDir) {
         getRootSpec().into(destDir);
         return this;
@@ -330,6 +346,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask into(Object destPath, Closure configureClosure) {
         getMainSpec().into(destPath, configureClosure);
         return this;
@@ -338,6 +355,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public CopySpec into(Object destPath, Action<? super CopySpec> copySpec) {
         getMainSpec().into(destPath, copySpec);
         return this;
@@ -346,6 +364,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask include(String... includes) {
         getMainSpec().include(includes);
         return this;
@@ -354,6 +373,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask include(Iterable<String> includes) {
         getMainSpec().include(includes);
         return this;
@@ -362,6 +382,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask include(Spec<FileTreeElement> includeSpec) {
         getMainSpec().include(includeSpec);
         return this;
@@ -370,6 +391,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask include(Closure includeSpec) {
         getMainSpec().include(includeSpec);
         return this;
@@ -378,6 +400,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask exclude(String... excludes) {
         getMainSpec().exclude(excludes);
         return this;
@@ -386,6 +409,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask exclude(Iterable<String> excludes) {
         getMainSpec().exclude(excludes);
         return this;
@@ -394,6 +418,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask exclude(Spec<FileTreeElement> excludeSpec) {
         getMainSpec().exclude(excludeSpec);
         return this;
@@ -402,6 +427,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask exclude(Closure excludeSpec) {
         getMainSpec().exclude(excludeSpec);
         return this;
@@ -410,6 +436,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask setIncludes(Iterable<String> includes) {
         getMainSpec().setIncludes(includes);
         return this;
@@ -419,6 +446,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
      * {@inheritDoc}
      */
     @Internal
+    @Override
     public Set<String> getIncludes() {
         return getMainSpec().getIncludes();
     }
@@ -426,6 +454,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask setExcludes(Iterable<String> excludes) {
         getMainSpec().setExcludes(excludes);
         return this;
@@ -435,6 +464,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
      * {@inheritDoc}
      */
     @Internal
+    @Override
     public Set<String> getExcludes() {
         return getMainSpec().getExcludes();
     }
@@ -442,14 +472,15 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask rename(final Closure closure) {
         return rename(new ClosureBackedTransformer(closure));
     }
 
     /**
      * {@inheritDoc}
-     * @param renamer
      */
+    @Override
     public AbstractCopyTask rename(Transformer<String, String> renamer) {
         getMainSpec().rename(renamer);
         return this;
@@ -458,6 +489,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask rename(String sourceRegEx, String replaceWith) {
         getMainSpec().rename(sourceRegEx, replaceWith);
         return this;
@@ -466,6 +498,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask rename(Pattern sourceRegEx, String replaceWith) {
         getMainSpec().rename(sourceRegEx, replaceWith);
         return this;
@@ -474,6 +507,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask filter(Map<String, ?> properties, Class<? extends FilterReader> filterType) {
         getMainSpec().filter(properties, filterType);
         return this;
@@ -482,6 +516,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask filter(Class<? extends FilterReader> filterType) {
         getMainSpec().filter(filterType);
         return this;
@@ -490,6 +525,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask filter(Closure closure) {
         getMainSpec().filter(closure);
         return this;
@@ -497,8 +533,8 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
 
     /**
      * {@inheritDoc}
-     * @param transformer
      */
+    @Override
     public AbstractCopyTask filter(Transformer<String, String> transformer) {
         getMainSpec().filter(transformer);
         return this;
@@ -507,6 +543,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask expand(Map<String, ?> properties) {
         getMainSpec().expand(properties);
         return this;
@@ -516,6 +553,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
      * {@inheritDoc}
      */
     @Internal
+    @Override
     public Integer getDirMode() {
         return getMainSpec().getDirMode();
     }
@@ -524,6 +562,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
      * {@inheritDoc}
      */
     @Internal
+    @Override
     public Integer getFileMode() {
         return getMainSpec().getFileMode();
     }
@@ -531,7 +570,8 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
-    public AbstractCopyTask setDirMode(Integer mode) {
+    @Override
+    public AbstractCopyTask setDirMode(@Nullable Integer mode) {
         getMainSpec().setDirMode(mode);
         return this;
     }
@@ -539,7 +579,8 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
-    public AbstractCopyTask setFileMode(Integer mode) {
+    @Override
+    public AbstractCopyTask setFileMode(@Nullable Integer mode) {
         getMainSpec().setFileMode(mode);
         return this;
     }
@@ -547,6 +588,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask eachFile(Action<? super FileCopyDetails> action) {
         getMainSpec().eachFile(action);
         return this;
@@ -555,6 +597,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public AbstractCopyTask eachFile(Closure closure) {
         getMainSpec().eachFile(closure);
         return this;
@@ -564,6 +607,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
      * {@inheritDoc}
      */
     @Internal
+    @Override
     public String getFilteringCharset() {
         return getMainSpec().getFilteringCharset();
     }
@@ -571,6 +615,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setFilteringCharset(String charset) {
         getMainSpec().setFilteringCharset(charset);
     }
