@@ -19,9 +19,9 @@ package org.gradle.api.internal.artifacts.ivyservice.resolveengine.result;
 import com.google.common.base.Objects;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Iterables;
+import org.gradle.api.Describable;
 import org.gradle.api.artifacts.result.ComponentSelectionCause;
 import org.gradle.api.artifacts.result.ComponentSelectionDescriptor;
-import org.gradle.api.artifacts.result.ComponentSelectionReason;
 
 import java.util.ArrayDeque;
 import java.util.Collections;
@@ -36,6 +36,7 @@ public class VersionSelectionReasons {
     public static final ComponentSelectionDescriptorInternal SELECTED_BY_RULE = new DefaultComponentSelectionDescriptor(ComponentSelectionCause.SELECTED_BY_RULE);
     public static final ComponentSelectionDescriptorInternal COMPOSITE_BUILD = new DefaultComponentSelectionDescriptor(ComponentSelectionCause.COMPOSITE_BUILD);
     public static final ComponentSelectionDescriptorInternal CONSTRAINT = new DefaultComponentSelectionDescriptor(ComponentSelectionCause.CONSTRAINT);
+    public static final ComponentSelectionDescriptorInternal REJECTION = new DefaultComponentSelectionDescriptor(ComponentSelectionCause.REJECTION);
 
     public static ComponentSelectionReasonInternal requested() {
         return new DefaultComponentSelectionReason(REQUESTED);
@@ -45,7 +46,7 @@ public class VersionSelectionReasons {
         return new DefaultComponentSelectionReason(Collections.<ComponentSelectionDescriptor>emptyList());
     }
 
-    public static ComponentSelectionReason root() {
+    public static ComponentSelectionReasonInternal root() {
         return new DefaultComponentSelectionReason(ROOT);
     }
 
@@ -114,7 +115,7 @@ public class VersionSelectionReasons {
         }
 
         @Override
-        public ComponentSelectionReasonInternal addCause(ComponentSelectionCause cause, String description) {
+        public ComponentSelectionReasonInternal addCause(ComponentSelectionCause cause, Describable description) {
             addCause(new DefaultComponentSelectionDescriptor(cause, description));
             return this;
         }
@@ -129,13 +130,9 @@ public class VersionSelectionReasons {
 
         @Override
         public ComponentSelectionReasonInternal addCause(ComponentSelectionDescriptor description) {
-            if (!descriptions.contains(description)) {
-                ComponentSelectionCause cause = description.getCause();
-                if (descriptions.isEmpty() && cause != ComponentSelectionCause.REQUESTED && cause != ComponentSelectionCause.ROOT) {
-                    // initial reason must always be either root or requested
-                    descriptions.add(REQUESTED);
-                }
-                descriptions.add((ComponentSelectionDescriptorInternal) description);
+            ComponentSelectionDescriptorInternal descriptor = (ComponentSelectionDescriptorInternal) description;
+            if (!descriptions.contains(descriptor)) {
+                descriptions.add(descriptor);
             }
             return this;
         }
