@@ -25,13 +25,15 @@ import org.gradle.api.internal.plugins.DslObject;
 import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.internal.tasks.TaskContainerInternal;
 import org.gradle.api.reporting.components.ComponentReport;
+import org.gradle.api.reporting.dependents.DependentComponentsReport;
 import org.gradle.api.reporting.model.ModelReport;
-import org.gradle.api.tasks.diagnostics.*;
+import org.gradle.api.tasks.diagnostics.BuildEnvironmentReportTask;
+import org.gradle.api.tasks.diagnostics.DependencyInsightReportTask;
+import org.gradle.api.tasks.diagnostics.DependencyReportTask;
+import org.gradle.api.tasks.diagnostics.ProjectReportTask;
+import org.gradle.api.tasks.diagnostics.PropertyReportTask;
+import org.gradle.api.tasks.diagnostics.TaskReportTask;
 import org.gradle.configuration.Help;
-import org.gradle.internal.service.ServiceRegistry;
-import org.gradle.model.Defaults;
-import org.gradle.model.Path;
-import org.gradle.model.RuleSource;
 
 import java.util.concurrent.Callable;
 
@@ -47,6 +49,7 @@ public class HelpTasksPlugin implements Plugin<ProjectInternal> {
     public static final String DEPENDENCY_INSIGHT_TASK = "dependencyInsight";
     public static final String COMPONENTS_TASK = "components";
     public static final String MODEL_TASK = "model";
+    public static final String DEPENDENT_COMPONENTS_TASK = "dependentComponents";
 
     @Override
     public void apply(final ProjectInternal project) {
@@ -63,18 +66,7 @@ public class HelpTasksPlugin implements Plugin<ProjectInternal> {
         tasks.addPlaceholderAction(BuildEnvironmentReportTask.TASK_NAME, BuildEnvironmentReportTask.class, new BuildEnvironmentReportTaskAction(projectName));
         tasks.addPlaceholderAction(COMPONENTS_TASK, ComponentReport.class, new ComponentReportAction(projectName));
         tasks.addPlaceholderAction(MODEL_TASK, ModelReport.class, new ModelReportAction(projectName));
-    }
-
-    static class Rules extends RuleSource {
-        @Defaults
-        void addDefaultDependenciesReportConfiguration(@Path("tasks.dependencyInsight") DependencyInsightReportTask task, final ServiceRegistry services) {
-            new DslObject(task).getConventionMapping().map("configuration", new Callable<Object>() {
-                public Object call() {
-                    BuildableJavaComponent javaProject = services.get(ComponentRegistry.class).getMainComponent();
-                    return javaProject == null ? null : javaProject.getCompileDependencies();
-                }
-            });
-        }
+        tasks.addPlaceholderAction(DEPENDENT_COMPONENTS_TASK, DependentComponentsReport.class, new DependentComponentsReportAction(projectName));
     }
 
     private static class HelpAction implements Action<Help> {
@@ -151,6 +143,12 @@ public class HelpTasksPlugin implements Plugin<ProjectInternal> {
             task.setDescription("Displays the insight into a specific dependency in " + projectName + ".");
             task.setGroup(HELP_GROUP);
             task.setImpliesSubProjects(true);
+            new DslObject(task).getConventionMapping().map("configuration", new Callable<Object>() {
+                public Object call() {
+                    BuildableJavaComponent javaProject = ((ProjectInternal) task.getProject()).getServices().get(ComponentRegistry.class).getMainComponent();
+                    return javaProject == null ? null : javaProject.getCompileDependencies();
+                }
+            });
         }
     }
 
@@ -213,4 +211,19 @@ public class HelpTasksPlugin implements Plugin<ProjectInternal> {
             task.setImpliesSubProjects(true);
         }
     }
+
+    private static class DependentComponentsReportAction implements Action<DependentComponentsReport> {
+        private final String projectName;
+
+        public DependentComponentsReportAction(String projectName) {
+            this.projectName = projectName;
+        }
+
+        @Override
+        public void execute(DependentComponentsReport task) {
+            task.setDescription("Displays the dependent components of components in " + projectName + ". [incubating]");
+            task.setGroup(HELP_GROUP);
+            task.setImpliesSubProjects(true);
+        }
+    }
 }
