@@ -25,7 +25,8 @@ import java.io.File;
  * Provides values that are set during the build, or defaulted when not running in a build context (e.g. IDE).
  */
 public class IntegrationTestBuildContext {
-
+    // Collect this early, as the process' current directory can change during embedded test execution
+    public static final TestFile TEST_DIR = new TestFile(new File(".").toURI());
     public static final IntegrationTestBuildContext INSTANCE = new IntegrationTestBuildContext();
 
     public TestFile getGradleHomeDir() {
@@ -36,14 +37,6 @@ public class IntegrationTestBuildContext {
         return file("integTest.samplesdir", String.format("%s/samples", getGradleHomeDir()));
     }
 
-    public TestFile getUserGuideOutputDir() {
-        return file("integTest.userGuideOutputDir", "subprojects/docs/src/samples/userguideOutput");
-    }
-
-    public TestFile getUserGuideInfoDir() {
-        return file("integTest.userGuideInfoDir", "subprojects/docs/build/src");
-    }
-
     public TestFile getDistributionsDir() {
         return file("integTest.distsDir", "build/distributions");
     }
@@ -98,17 +91,25 @@ public class IntegrationTestBuildContext {
             return new UnderDevelopmentGradleDistribution();
         }
         TestFile previousVersionDir = getGradleUserHomeDir().getParentFile().file("previousVersion");
-        if(version.startsWith("#")){
+        if (version.startsWith("#")) {
             return new BuildServerGradleDistribution(version, previousVersionDir.file(version));
         }
         return new ReleasedGradleDistribution(version, previousVersionDir.file(version));
     }
 
     protected static TestFile file(String propertyName, String defaultFile) {
-        String path = System.getProperty(propertyName, defaultFile);
+        String defaultPath;
+        if (defaultFile == null) {
+            defaultPath = null;
+        } else if (new File(defaultFile).isAbsolute()) {
+            defaultPath = defaultFile;
+        } else {
+            defaultPath = TEST_DIR.file(defaultFile).getAbsolutePath();
+        }
+        String path = System.getProperty(propertyName, defaultPath);
         if (path == null) {
             throw new RuntimeException(String.format("You must set the '%s' property to run the integration tests. The default passed was: '%s'",
-                    propertyName, defaultFile));
+                propertyName, defaultFile));
         }
         return new TestFile(new File(path));
     }
