@@ -20,6 +20,7 @@ import org.gradle.api.Incubating;
 import org.gradle.api.file.RegularFile;
 import org.gradle.api.provider.Provider;
 import org.gradle.language.ComponentWithOutputs;
+import org.gradle.api.component.PublishableComponent;
 import org.gradle.language.nativeplatform.ComponentWithExecutable;
 import org.gradle.language.nativeplatform.ComponentWithInstallation;
 import org.gradle.language.nativeplatform.ComponentWithRuntimeUsage;
@@ -30,7 +31,7 @@ import org.gradle.language.nativeplatform.ComponentWithRuntimeUsage;
  * @since 4.2
  */
 @Incubating
-public interface CppExecutable extends CppBinary, ComponentWithExecutable, ComponentWithInstallation, ComponentWithOutputs, ComponentWithRuntimeUsage {
+public interface CppExecutable extends CppBinary, ComponentWithExecutable, ComponentWithInstallation, ComponentWithOutputs, ComponentWithRuntimeUsage, PublishableComponent {
     /**
      * Returns the executable file to use with a debugger for this executable.
      *
