@@ -60,8 +60,8 @@ public class DefaultCppBinary extends DefaultNativeBinary implements CppBinary {
     private final Property<CppCompile> compileTaskProperty;
     private final NativeVariantIdentity identity;
 
-    public DefaultCppBinary(String name, ProjectLayout projectLayout, ObjectFactory objects, Provider<String> baseName, FileCollection sourceFiles, FileCollection componentHeaderDirs, ConfigurationContainer configurations, Configuration componentImplementation, CppPlatform targetPlatform, NativeToolChainInternal toolChain, PlatformToolProvider platformToolProvider, NativeVariantIdentity identity) {
-        super(name, objects, projectLayout, componentImplementation);
+    public DefaultCppBinary(Names names, ProjectLayout projectLayout, ObjectFactory objects, Provider<String> baseName, FileCollection sourceFiles, FileCollection componentHeaderDirs, ConfigurationContainer configurations, Configuration componentImplementation, CppPlatform targetPlatform, NativeToolChainInternal toolChain, PlatformToolProvider platformToolProvider, NativeVariantIdentity identity) {
+        super(names, objects, projectLayout, componentImplementation);
         this.baseName = baseName;
         this.sourceFiles = sourceFiles;
         this.targetPlatform = targetPlatform;
@@ -70,8 +70,6 @@ public class DefaultCppBinary extends DefaultNativeBinary implements CppBinary {
         this.compileTaskProperty = objects.property(CppCompile.class);
         this.identity = identity;
 
-        Names names = getNames();
-
         // TODO - reduce duplication with Swift binary
 
         Configuration includePathConfig = configurations.create(names.withPrefix("cppCompile"));
