@@ -16,9 +16,9 @@
 package org.gradle.language.nativeplatform.internal.incremental;
 
 import com.google.common.base.Objects;
-import org.gradle.api.internal.changedetection.state.FileSnapshot;
 import org.gradle.api.internal.changedetection.state.FileSystemSnapshotter;
 import org.gradle.internal.file.FileType;
+import org.gradle.internal.hash.HashCode;
 import org.gradle.language.nativeplatform.internal.Expression;
 import org.gradle.language.nativeplatform.internal.Include;
 import org.gradle.language.nativeplatform.internal.IncludeDirectives;
@@ -350,8 +350,8 @@ public class DefaultSourceIncludesResolver implements SourceIncludesResolver {
             }
 
             File candidate = new File(searchDir, includePath);
-            FileSnapshot fileSnapshot = fileSystemSnapshotter.snapshotSelf(candidate);
-            includeFile = fileSnapshot.getType() == FileType.RegularFile ? new SystemIncludeFile(candidate, includePath, fileSnapshot) : MISSING_INCLUDE_FILE;
+            HashCode contentHash = fileSystemSnapshotter.getRegularFileContentHash(candidate);
+            includeFile = contentHash != null ? new SystemIncludeFile(candidate, includePath, contentHash) : MISSING_INCLUDE_FILE;
             contents.put(includePath, includeFile);
             return includeFile;
         }
@@ -381,12 +381,12 @@ public class DefaultSourceIncludesResolver implements SourceIncludesResolver {
     private static class SystemIncludeFile extends CachedIncludeFile implements IncludeFile {
         final File file;
         final String includePath;
-        final FileSnapshot snapshot;
+        final HashCode contentHash;
 
-        SystemIncludeFile(File file, String includePath, FileSnapshot snapshot) {
+        SystemIncludeFile(File file, String includePath, HashCode contentHash) {
             this.file = file;
             this.includePath = includePath;
-            this.snapshot = snapshot;
+            this.contentHash = contentHash;
         }
 
         @Override
@@ -406,12 +406,12 @@ public class DefaultSourceIncludesResolver implements SourceIncludesResolver {
 
         @Override
         FileType getType() {
-            return snapshot.getType();
+            return FileType.RegularFile;
         }
 
         @Override
-        public FileSnapshot getSnapshot() {
-            return snapshot;
+        public HashCode getContentHash() {
+            return contentHash;
         }
 
         @Override
@@ -423,24 +423,24 @@ public class DefaultSourceIncludesResolver implements SourceIncludesResolver {
                 return false;
             }
             SystemIncludeFile other = (SystemIncludeFile) obj;
-            return Objects.equal(file, other.file) && snapshot.equals(other.snapshot);
+            return Objects.equal(file, other.file) && contentHash.equals(other.contentHash);
         }
 
         @Override
         public int hashCode() {
-            return snapshot.hashCode();
+            return contentHash.hashCode();
         }
 
         IncludeFile toIncludeFile(boolean quotedPath) {
             if (quotedPath) {
-                return new QuotedIncludeFile(file, includePath, snapshot);
+                return new QuotedIncludeFile(file, includePath, contentHash);
             }
             return this;
         }
 
         private static class QuotedIncludeFile extends SystemIncludeFile {
-            QuotedIncludeFile(File file, String includePath, FileSnapshot snapshot) {
-                super(file, includePath, snapshot);
+            QuotedIncludeFile(File file, String includePath, HashCode contentHash) {
+                super(file, includePath, contentHash);
             }
 
             @Override
