@@ -25,8 +25,9 @@ import org.gradle.api.JavaVersion;
 import org.gradle.api.Project;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.file.FileTree;
-import org.gradle.api.internal.tasks.compile.processing.AnnotationProcessorPathFactory;
 import org.gradle.api.internal.tasks.compile.CompilerForkUtils;
+import org.gradle.api.internal.tasks.compile.processing.AnnotationProcessorPathFactory;
+import org.gradle.api.internal.tasks.scala.CleaningScalaCompiler;
 import org.gradle.api.internal.tasks.scala.DefaultScalaJavaJointCompileSpec;
 import org.gradle.api.internal.tasks.scala.DefaultScalaJavaJointCompileSpecFactory;
 import org.gradle.api.internal.tasks.scala.ScalaCompileSpec;
@@ -43,6 +44,7 @@ import org.gradle.api.tasks.PathSensitivity;
 import org.gradle.api.tasks.TaskAction;
 import org.gradle.api.tasks.compile.AbstractCompile;
 import org.gradle.api.tasks.compile.CompileOptions;
+import org.gradle.language.base.internal.compile.Compiler;
 
 import java.io.File;
 import java.util.Collections;
@@ -89,7 +91,20 @@ public abstract class AbstractScalaCompile extends AbstractCompile {
     protected void compile() {
         ScalaJavaJointCompileSpec spec = createSpec();
         configureIncrementalCompilation(spec);
-        getCompiler(spec).execute(spec);
+        Compiler<ScalaJavaJointCompileSpec> compiler = getCompiler(spec);
+        if (isNonIncrementalCompilation()) {
+            compiler = new CleaningScalaCompiler(compiler, getOutputs());
+        }
+        compiler.execute(spec);
+    }
+
+    private boolean isNonIncrementalCompilation() {
+        File analysisFile = getScalaCompileOptions().getIncrementalOptions().getAnalysisFile();
+        if (analysisFile != null && !analysisFile.exists()) {
+            LOGGER.info("Zinc is doing a full recompile since the analysis file doesn't exist");
+            return true;
+        }
+        return false;
     }
 
     protected ScalaJavaJointCompileSpec createSpec() {
