@@ -17,18 +17,20 @@
 package org.gradle.tooling.internal.provider;
 
 import org.gradle.api.execution.internal.TaskInputsListener;
-import org.gradle.initialization.GradleLauncherFactory;
 import org.gradle.internal.classpath.CachedClasspathTransformer;
 import org.gradle.internal.concurrent.ExecutorFactory;
 import org.gradle.internal.concurrent.ParallelismConfigurationManager;
+import org.gradle.internal.filewatch.DefaultFileSystemChangeWaiterFactory;
+import org.gradle.internal.filewatch.FileSystemChangeWaiterFactory;
 import org.gradle.internal.filewatch.FileWatcherFactory;
 import org.gradle.internal.invocation.BuildActionRunner;
 import org.gradle.internal.logging.LoggingManagerInternal;
 import org.gradle.internal.logging.text.StyledTextOutputFactory;
-import org.gradle.internal.progress.BuildOperationListenerManager;
+import org.gradle.internal.operations.BuildOperationListenerManager;
 import org.gradle.internal.service.ServiceRegistration;
 import org.gradle.internal.service.scopes.AbstractPluginServiceRegistry;
 import org.gradle.internal.service.scopes.GradleUserHomeScopeServiceRegistry;
+import org.gradle.internal.time.Time;
 import org.gradle.launcher.exec.BuildExecuter;
 import org.gradle.launcher.exec.BuildTreeScopeBuildActionExecuter;
 import org.gradle.launcher.exec.ChainingBuildActionRunner;
@@ -58,15 +60,15 @@ public class LauncherServices extends AbstractPluginServiceRegistry {
     static class ToolingGlobalScopeServices {
         BuildExecuter createBuildExecuter(List<BuildActionRunner> buildActionRunners,
                                           List<SubscribableBuildActionRunnerRegistration> registrations,
-                                          GradleLauncherFactory gradleLauncherFactory,
                                           BuildOperationListenerManager buildOperationListenerManager,
-                                          FileWatcherFactory fileWatcherFactory,
                                           TaskInputsListener inputsListener,
                                           StyledTextOutputFactory styledTextOutputFactory,
                                           ExecutorFactory executorFactory,
                                           LoggingManagerInternal loggingManager,
                                           GradleUserHomeScopeServiceRegistry userHomeServiceRegistry,
-                                          ParallelismConfigurationManager parallelismConfigurationManager) {
+                                          FileSystemChangeWaiterFactory fileSystemChangeWaiterFactory,
+                                          ParallelismConfigurationManager parallelismConfigurationManager
+        ) {
             return new SetupLoggingActionExecuter(
                 new SessionFailureReportingActionExecuter(
                     new StartParamsValidatingActionExecuter(
@@ -75,20 +77,28 @@ public class LauncherServices extends AbstractPluginServiceRegistry {
                                 new ServicesSetupBuildActionExecuter(
                                     new ContinuousBuildActionExecuter(
                                         new BuildTreeScopeBuildActionExecuter(
-                                            new InProcessBuildActionExecuter(gradleLauncherFactory,
+                                            new InProcessBuildActionExecuter(
                                                 new SubscribableBuildActionRunner(
                                                     new RunAsBuildOperationBuildActionRunner(
                                                         new ValidatingBuildActionRunner(
                                                             new ChainingBuildActionRunner(buildActionRunners))),
-                                                    buildOperationListenerManager, registrations))),
-                                        fileWatcherFactory,
+                                                    buildOperationListenerManager,
+                                                    registrations)
+                                            )
+                                        ),
+                                        fileSystemChangeWaiterFactory,
                                         inputsListener,
                                         styledTextOutputFactory,
                                         executorFactory),
                                     userHomeServiceRegistry)),
                             parallelismConfigurationManager)),
-                    styledTextOutputFactory),
-                loggingManager, parallelismConfigurationManager);
+                    styledTextOutputFactory,
+                    Time.clock()),
+                loggingManager);
+        }
+
+        FileSystemChangeWaiterFactory createFileSystemChangeWaiterFactory(FileWatcherFactory fileWatcherFactory) {
+            return new DefaultFileSystemChangeWaiterFactory(fileWatcherFactory);
         }
 
         ExecuteBuildActionRunner createExecuteBuildActionRunner() {
